searchNodes=[{"ref":"DataLogger.html","title":"DataLogger","type":"module","doc":"A DataLogger can log any data to any configured destination. A destination can be configured using the the application configuration: config :data_logger, destinations: [ {DestinationImplementation, %{option_one: value_one, option_two: value_two}}, {AnotherDestinationImplementation, %{option: value}} ] When such a configuration is defined, chunks of data, represented by Elixir terms can be logged to them by using the DataLogger.log/2 function. For example we could have two schemas in a relational database : green and red. We would like to send a list of records to a destination representing this database. When we have data that should go to the green schema, we would use: DataLogger.log(:green, [row1, row2, row3]) When we want data sent and stored to the red schema, we would use: DataLogger.log(:red, [row1, row2, row3, row4]) This way we could have different schemas or tables or clients, etc. and send data related to them to a storage defined for them. In the red and green example the configuration would be: config :data_logger, destinations: [ {RelationalDBDestination, %{host: &quot;localhost&quot;, user: &quot;inflowmatix&quot;, password: &quot;secret&quot;}} ] The destination should be a module, which implements the DataLogger.Destination protocol. For both the green and the red data there will be independent supervision tree with a worker per destination so the data sent to the green destination won&#39;t be in the way of the data sent to the red destination. By default the data logged by DataLogger.log/2 is sent in the worker process for the given topic (green or red) in the above example. This can be changed if in the options of the destination :send_async is set to true: config :data_logger, destinations: [ {RelationalDBDestination, %{host: &quot;localhost&quot;, user: &quot;inflowmatix&quot;, password: &quot;secret&quot;, send_async: true}} ] Now every chunk of data logged to that topic will be sent in its own supervised process. The DataLogger.Destination behaviour implementation can define on_error/4 or/and on_success/4 callbacks so the result can be handled. Ensuring that the data has been sent and retrying sending it, if needed is a responsibility of the destination implementation."},{"ref":"DataLogger.html#log/2","title":"DataLogger.log/2","type":"function","doc":"This function is the sole entry point of the DataLogger application. It is used to log/send the data passed to it to the configured destinations. The topic given can be used to send the data to different sub-destinations of every destination configured."},{"ref":"DataLogger.Application.html","title":"DataLogger.Application","type":"module","doc":"The application module of the DataLogger application. It starts a registry for registered processes, a registry for broadcasting of messages and the main dynamic supervisor, responsible for adding new logger sub-trees per sub-destination and destination. It could start a task supervisor if :send_async is set to true for any configured destination."},{"ref":"DataLogger.Application.html#start/2","title":"DataLogger.Application.start/2","type":"function","doc":"Called when an application is started. This function is called when an application is started using Application.start/2 (and functions on top of that, such as Application.ensure_started/2). This function should start the top-level process of the application (which should be the top supervisor of the application&#39;s supervision tree if the application follows the OTP design principles around supervision). start_type defines how the application is started: :normal - used if the startup is a normal startup or if the application is distributed and is started on the current node because of a failover from another node and the application specification key :start_phases is :undefined. {:takeover, node} - used if the application is distributed and is started on the current node because of a failover on the node node. {:failover, node} - used if the application is distributed and is started on the current node because of a failover on node node, and the application specification key :start_phases is not :undefined. start_args are the arguments passed to the application in the :mod specification key (e.g., mod: {MyApp, [:my_args]}). This function should either return {:ok, pid} or {:ok, pid, state} if startup is successful. pid should be the PID of the top supervisor. state can be an arbitrary term, and if omitted will default to []; if the application is later stopped, state is passed to the stop/1 callback (see the documentation for the c:stop/1 callback for more information). use Application provides no default implementation for the start/2 callback. Callback implementation for Application.start/2."},{"ref":"DataLogger.Destination.html","title":"DataLogger.Destination","type":"behaviour","doc":"A behaviour, representing a destination for data logging. The mandatory callback to implement is DataLogger.Destination.send_data/3. An implementation should handle errors and retries by using the optional callbacks DataLogger.Destination.on_error/4 or/and DataLogger.Destination.on_success/4. These functions are called with the result of the call to the DataLogger.Destination.send_data/4 function. Also an implementation can have some custom initialization, using the optional callback DataLogger.Destination.init/1 which gets the configured options from the config and can return modified options. By default it returns what is passed to it. A possible implementation could look like this: defmodule RelationalDBDestination do use DataLogger.Destination @impl true def send_data(topic, data, options) do connection = ConnectionToDBImpl.connect(options) query_to_insert_data = transform_data_to_query(topic, data) case ConnectionToDBImpl.execute(connection, query_to_insert_data, data) do :ok -&gt; :ok {:error, reason} -&gt; {:error, reason} end end end The implementation can also define the DataLogger.Destination.on_error/4 function and retry on error or log some message. The default implementations of the DataLogger.Destination.on_error/4 and DataLogger.Destination.on_success/4 callbacks, do nothing. The above example implementation can be configured in the application configuration like this: config :data_logger, destinations: [ {RelationalDBDestination, %{host: &quot;localhost&quot;, user: &quot;inflowmatix&quot;, password: &quot;secret&quot;, send_async: true}} ]"},{"ref":"DataLogger.Destination.html#c:initialize/1","title":"DataLogger.Destination.initialize/1","type":"callback","doc":""},{"ref":"DataLogger.Destination.html#c:on_error/4","title":"DataLogger.Destination.on_error/4","type":"callback","doc":""},{"ref":"DataLogger.Destination.html#c:on_success/4","title":"DataLogger.Destination.on_success/4","type":"callback","doc":""},{"ref":"DataLogger.Destination.html#c:send_data/3","title":"DataLogger.Destination.send_data/3","type":"callback","doc":""},{"ref":"DataLogger.Destination.html#t:options/0","title":"DataLogger.Destination.options/0","type":"type","doc":""},{"ref":"DataLogger.Destination.html#t:prefix/0","title":"DataLogger.Destination.prefix/0","type":"type","doc":""},{"ref":"DataLogger.Destination.html#t:send_result/0","title":"DataLogger.Destination.send_result/0","type":"type","doc":""},{"ref":"DataLogger.Destination.html#t:topic/0","title":"DataLogger.Destination.topic/0","type":"type","doc":""},{"ref":"DataLogger.Destination.Controller.html","title":"DataLogger.Destination.Controller","type":"module","doc":"A worker process, created and supervised per destination and per topic/sub-destination. The first time DataLogger.log/2 is called with a given topic DataLogger.Destination.Controllers for this topic for every configured destination are created. They are supervised by a new supervisor, created for the given topic. A DataLogger.Destination.Controller process is registered in a pub-sub registry with its topic, so when data is sent to the topic, every such process is notified and data is casted to it in the form of {:log_data, topic, data}. If the destination of a DataLogger.Destination.Controller is configured with send_async: true, the process will be creating a task per cast and will be responsible for invoking the on_error/4/on_success/4 callbacks of the destination when the task finishes. It will also react when the task is :DOWN."},{"ref":"DataLogger.Destination.Controller.html#child_spec/1","title":"DataLogger.Destination.Controller.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"DataLogger.Destination.Supervisor.html","title":"DataLogger.Destination.Supervisor","type":"module","doc":"Supervisor of a group of DataLogger.Destination.Controller workers for given topic. For every configured destination, there will be a worker (unless prefixes are used). For example if we configured a NoSQL destination and a relational destination, for given document/schema used as topic a new DataLogger.Destination.Supervisor will be created and it will be supervising two DataLogger.Destination.Controller workers. If the destinations are specifying prefixes, by including the option prefix: &lt;prefix&gt;, the supervisor will create and supervise loggers for only these destinations, that have prefix which prefixes the topic given to start_link. If the destinations are: destinations: [ {MemoryDestination, %{destination: 1, prefix: :blue}}, {MemoryDestination, %{destination: 2, prefix: :purple}} ] And the supervisor is started with topic of &quot;purple_1&quot; it will only start and supervise a proces for {MemoryDestination, %{destination: 2, prefix: :purple}}. Using destinations with prefixes we could send part of our data only to subset of the configured destinations."},{"ref":"DataLogger.Destination.Supervisor.html#child_spec/1","title":"DataLogger.Destination.Supervisor.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"DataLogger.Supervisor.html","title":"DataLogger.Supervisor","type":"module","doc":"A DynamicSupervisor which adds a new DataLogger.Destination.Supervisor and its sub-tree of workers to its children, whenever a new topic, for which data haven&#39;t been logged until now is passed to DataLogger.log/2. Using this, the DataLogger application is building its supervision tree lazy and on demand."},{"ref":"readme.html","title":"DataLogger","type":"extras","doc":"DataLogger A logger that can be used to log any kind of data to remote or local destinations. It is similar to the Elixir Logger, but doesn&#39;t log only binary data and is highly concurrent, supporting logging to multiple destinations concurrently, even being able to log to multiple sub-destinations of the same destination in the same time. Can be used to send data to persistent queues, which can be then consumed by on-demand consumers powered by GenStage if the Elixir producer doesn&#39;t work on demand. Can be used to send data to data bases, transofrming it in the process. Databases like Postgres, MySql, DynamoDB, MongoDB, etc. Can be used as a replacement of the Elixir Logger too, if a smart wrapper around it is built. Can be used to send data to pub-sub topics."},{"ref":"readme.html#installation","title":"DataLogger - Installation","type":"extras","doc":"The package can be installed by adding data_logger to your list of dependencies in mix.exs: def deps do [ {:data_logger, &quot;~&gt; 0.3.0&quot;} ] end Run mix deps.get to download it."},{"ref":"readme.html#usage","title":"DataLogger - Usage","type":"extras","doc":"A DataLogger can log any data to any configured destination. A destination can be configured using the the application configuration: config :data_logger, destinations: [ {DestinationImplementation, %{option_one: value_one, option_two: value_two}}, {AnoterDestinationImplementation, %{option: value}} ] When such a configuration is defined, chunks of data, represented by Elixir terms can be logged to them by using the DataLogger.log/2 function. For example we could have two schemas in a relational database : green and red. We would like to send a list of records to a destination representing this database. When we have data that should go to the green schema, we would use: DataLogger.log(:green, [row1, row2, row3]) When we want data sent and stored to the red schema, we would use: DataLogger.log(:red, [row1, row2, row3, row4]) This way we could have different schemas or tables or clients, etc and send data related to them to a storage defined for them. In the red and green example the configuration would be: config :data_logger, destinations: [ {RelationalDBDestination, %{host: &quot;localhost&quot;, user: &quot;inflowmatix&quot;, password: &quot;secret&quot;}} ] The destination should be a module, which implements the DataLogger.Destination protocol. For both the green and the red data there will be independent supervisor tree with a worker per destination so the data sent to the green destination won&#39;t be in the way of the data sent to the red destination. By default the data logged by DataLogger.log/2 is send in the worker process for the given topic (green or red) in the above example. This can be changed if in the options of the destination :send_async is set to true: config :data_logger, destinations: [ {RelationalDBDestination, %{host: &quot;localhost&quot;, user: &quot;inflowmatix&quot;, password: &quot;secret&quot;, send_async: true}} ] Now every chunk of data logged with that topic will be send in its own supervised process. The DataLogger.Destination behaviour implementation can define on_error/4 or/and on_success/4 callbacks so the result can be handled. Ensuring that the data has been sent and retrying sending it, if needed is a responsibility of the destination implementation. Prefixes From version 0.3.0 the DataLogger can have destinations with prefixes, like: destinations: [ {MyDestination, %{prefix: &quot;blue&quot;}}, {MyOtherDestination, %{prefix: &quot;purple&quot;}} ] Now data logged with DataLogger.log(&quot;purple_1&quot;, &lt;data&gt;) will be sent only the MyOtherDestination. Multiple destinations can have the same prefix."},{"ref":"readme.html#documentation","title":"DataLogger - Documentation","type":"extras","doc":"Documentation can be found at https://hexdocs.pm/data_logger."}]